<?php

namespace Drupal\activitypub\Form;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

/**
 * ActivityPub Activity form.
 *
 * @property \Drupal\activitypub\Entity\ActivityPubActivityInterface $entity
 */
class ActivityPubActivityForm extends ContentEntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    if ($this->entity->isNew()) {
      $form['#title'] = $this->t('Add activity');
    }
    else {
      $form['#title'] = $this->t('Edit activity @id', ['@id' => $this->entity->id()]);
    }

    $form['type'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Type'),
      '#default_value' => $this->entity->getType(),
    ];

    $form['processed'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Processed'),
      '#default_value' => $this->entity->isProcessed(),
    ];

    $form['status'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Status'),
      '#default_value' => $this->entity->isPublished(),
    ];

    $form['queued'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Queued'),
      '#default_value' => $this->entity->isQueued(),
      '#description' => $this->t('This is managed when added or removed from the queue, so only change this when necessary.')
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $result = parent::save($form, $form_state);
    $message_args = ['%label' => $this->entity->label()];
    $message = $result == SAVED_NEW
      ? $this->t('Created new ActivityPub activity %label.', $message_args)
      : $this->t('Updated ActivityPub activity %label.', $message_args);
    $this->messenger()->addStatus($message);
    Cache::invalidateTags(['user:' . $this->entity->getOwnerId()]);
    $form_state->setRedirectUrl(Url::fromRoute('activitypub.user.activities', ['user' => $this->entity->getOwnerId()]));
    return $result;
  }

}
