<?php

namespace Drupal\activitypub\Services;

use Drupal\activitypub\Entity\ActivityPubActorInterface;
use Drupal\user\UserInterface;

interface ActivityPubUtilityInterface {

  /**
   * Returns the ActivityPub name.
   *
   * @param \Drupal\activitypub\Entity\ActivityPubActorInterface $actor
   *
   * @return string
   */
  public function getActivityPubName(ActivityPubActorInterface $actor);

  /**
   * Returns the ActivityPub ID based on the Actor.
   *
   * @param \Drupal\activitypub\Entity\ActivityPubActorInterface $actor
   *
   * @return string
   */
  public function getActivityPubID(ActivityPubActorInterface $actor);

  /**
   * Returns the user avatar, if any.
   *
   * @param \Drupal\user\UserInterface $user
   *
   * @return string
   */
  public function getActivityPubUserAvatar(UserInterface $user);

  /**
   * Get the ActivityPubServer.
   *
   * @param array $config
   *
   * @return \ActivityPhp\Server
   */
  public function getServer(array $config = []);

  /**
   * Alter the nodeinfo data.
   *
   * @param array $data
   */
  public function alterNodeInfo(array &$data);

  /**
   * Act on user delete.
   *
   * @param \Drupal\user\UserInterface $account
   */
  public function onUserDelete(UserInterface $account);

  /**
   * Act on actor delete.
   *
   * @param \Drupal\activitypub\Entity\ActivityPubActorInterface $actor
   */
  public function onActorDelete(ActivityPubActorInterface $actor);

}
