<?php

namespace Drupal\activitypub\Services\Type;

use Drupal\activitypub\Entity\ActivityPubActivityInterface;
use Drupal\Component\Plugin\ConfigurableInterface;
use Drupal\Component\Plugin\DependentPluginInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides an interface describing the activitypub types.
 */
interface TypePluginInterface extends ConfigurableInterface, DependentPluginInterface {

  /**
   * Returns whether to expose in type configuration form.
   *
   * @return bool
   */
  public function isExposed();

  /**
   * Returns the activities managed by this plugin.
   *
   * @return array
   *   A collection of objects.
   */
  public function getActivities();

  /**
   * Returns the objects managed by this plugin.
   *
   * @return array
   *   A collection of objects.
   */
  public function getObjects();

  /**
   * Returns the supported properties for an object.
   *
   * @param $object
   *
   * @return array
   *   A collection of objects.
   */
  public function getProperties($object);

  /**
   * Act on saving of an activity.
   *
   * @param \Drupal\activitypub\Entity\ActivityPubActivityInterface $activity
   *   The activity being saved.
   * @param $update
   *   Specifies whether the entity is being updated or created.
   */
  public function onActivitySave(ActivityPubActivityInterface $activity, $update = TRUE);

  /**
   * Provides a configuration form for this type.
   *
   * @param array $form
   *   A form API form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   *
   * @return array
   *   A form array.
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state);

  /**
   * Build the activity.
   *
   * @param \Drupal\activitypub\Entity\ActivityPubActivityInterface $activity
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *
   * @return array
   */
  public function build(ActivityPubActivityInterface $activity, EntityInterface $entity = NULL);

  /**
   * Do inbox processing.
   *
   * @param \Drupal\activitypub\Entity\ActivityPubActivityInterface $activity
   * @param \Drupal\Core\Entity\EntityInterface|null $entity
   */
  public function doInboxProcess(ActivityPubActivityInterface $activity, EntityInterface $entity = NULL);

}
