<?php

namespace Drupal\Tests\activitypub\Functional;

use Drupal\Core\Url;

/**
 * Tests Outbox functionality.
 *
 * @group activitypub
 */
class OutboxTest extends ActivityPubTestBase {

  /**
   * Test outbox responses.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Behat\Mink\Exception\ResponseTextException
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testOutboxResponse() {
    $assert_session = $this->assertSession();
    $page = $this->getSession()->getPage();

    $this->enableActivityPub($assert_session);
    $this->drupalLogout();

    $outbox = Url::fromRoute('activitypub.outbox', ['user' => $this->authenticatedUserOne->id(), 'activitypub_actor' => $this->accountNameOne], ['absolute' => TRUE])->toString();
    $this->drupalGet($outbox);
    $assert_session->statusCodeEquals(200);

    $content = json_decode($page->getContent());
    self::assertEquals("OrderedCollection", $content->type);
    $first_page = $content->first;
    $this->drupalGet($content->first);
    $assert_session->statusCodeEquals(200);
    $content = json_decode($page->getContent());
    self::assertEquals($first_page, $content->id);

    // Create type.
    $this->createType('Create', 'page', 'Note');

    // Send to outbox.
    $this->drupalLogin($this->authenticatedUserOne);
    $this->drupalGet('node/add/page');
    $assert_session->statusCodeEquals(200);
    $assert_session->pageTextContains('ActivityPub outbox');

    $edit = [
      'title[0][value]' => 'Page one',
      'body[0][value]' => 'Hello world!',
      'activitypub_type' => 'map',
    ];
    $this->drupalPostForm(NULL, $edit, 'Save');
    $assert_session->addressEquals('node/1');
    $this->drupalGet('node/1/edit');
    $assert_session->pageTextContains('This post has been sent to the outbox.');
    $this->drupalLogout();

    $this->drupalGet($outbox);
    $assert_session->statusCodeEquals(200);
    $content = json_decode($page->getContent());
    self::assertEquals(1, $content->totalItems);
    $this->drupalGet($content->first);
    $assert_session->statusCodeEquals(200);
    $content = json_decode($page->getContent());
    self::assertEquals(1, count($content->orderedItems));

    $this->drupalGet('nodeinfo/content');
    $content = json_decode($page->getContent());
    self::assertEquals(1, $content->usage->users->total);
    self::assertEquals(1, $content->usage->localPosts);

    $this->drupalLogin($this->authenticatedUserOne);
    $this->drupalGet('node/add/page');
    $edit = [
      'title[0][value]' => 'Page two',
      'body[0][value]' => 'Hello world!',
      'activitypub_type' => 'map',
    ];
    $this->drupalPostForm(NULL, $edit, 'Save');
    $assert_session->addressEquals('node/2');
    $this->drupalGet('node/2/edit');
    $assert_session->pageTextContains('This post has been sent to the outbox.');
    $this->drupalLogout();

    $this->drupalGet($outbox);
    $assert_session->statusCodeEquals(200);
    $content = json_decode($page->getContent());
    self::assertEquals(2, $content->totalItems);
    $this->drupalGet($content->first);
    $assert_session->statusCodeEquals(200);
    $content = json_decode($page->getContent());
    self::assertEquals(2, count($content->orderedItems));

    $this->drupalGet('nodeinfo/content', ['query' => ['page' => 1]]);
    $content = json_decode($page->getContent());
    self::assertEquals(1, $content->usage->users->total);
    self::assertEquals(2, $content->usage->localPosts);

  }

}
